# WIPBatch 1.0 by synesthetic.de
import os
import re
import c4d
from c4d import storage
from c4d import documents
from c4d import gui

PARAMETERS= 2000
PARAM1= 3000
PARAM2= 4000
PARAM3= 5000
OKCANCEL= 6000 
TEXT= 1000
UCAM= 1001
DIR= 1002
NTH= 1003
BACKUP= 1004
OK= 1010
CANCEL= 1011

def alphaNumKey(a):
    return [int(s) if s.isdigit() else s for s in re.split('([0-9]+)', a)]

class Dialog(gui.GeDialog):
    
  singlecam= True
  folder= None
  n= 1
  bc= None
  backup= True
   
  def CreateLayout(self):
      self.bc= c4d.BaseContainer()
      self.bc.SetData(c4d.FILENAME_DIRECTORY, True) 
      self.SetTitle("WIPBatch")
      self.GroupBegin(PARAMETERS, c4d.BFH_SCALEFIT, 1, 2)
      self.GroupBorderNoTitle(c4d.BORDER_GROUP_IN)
      self.GroupBorderSpace(left = 2, top=2, right=2, bottom=2)
      self.GroupBegin(PARAM1, c4d.BFH_SCALEFIT, 3, 1)
      self.AddStaticText(TEXT, c4d.BFH_SCALEFIT, 0, 0, "Source Directory", 0)
      self.filename = self.AddCustomGui(DIR, c4d.CUSTOMGUI_FILENAME, "Choose Directory", c4d.BFH_SCALEFIT, 400, 0, self.bc)
      self.AddCheckbox(BACKUP, c4d.BFH_SCALEFIT, 0, 0, "Use Backup Files")
      self.GroupEnd()
      self.GroupBegin(PARAM2, c4d.BFH_SCALEFIT, 3, 1)
      self.AddStaticText(TEXT, c4d.BFH_SCALEFIT|c4d.BFH_LEFT, 50, 0, "Use Nth Files", 0)
      self.AddEditNumber(NTH,c4d.BFH_SCALEFIT| c4d.BFH_LEFT, 20, 0)
      self.AddCheckbox(UCAM, c4d.BFH_SCALEFIT|c4d.BFH_RIGHT, 0, 0, "Use Unified Camera")
      self.GroupEnd()
      self.GroupBegin(OKCANCEL, c4d.BFH_SCALEFIT, 2, 1)
      self.GroupBorderNoTitle(c4d.BORDER_GROUP_IN)
      self.GroupBorderSpace(left = 2, top=2, right=2, bottom=2)
      self.AddButton(OK, c4d.BFH_SCALE, name="OK")
      self.AddButton(CANCEL, c4d.BFH_SCALE, name="Cancel")
      self.GroupEnd() 
      self.SetLong(NTH, 1)
      self.SetBool(UCAM, True)
      self.SetBool(BACKUP, True)

      return True

  def Command(self, id, msg): 
    if id == DIR: self.folder= self.GetFilename(DIR)
    if id == UCAM: self.singlecam= self.GetBool(UCAM)
    if id == BACKUP: self.backup= self.GetBool(BACKUP)
    if id == NTH: self.n= self.GetLong(NTH)
    if id == CANCEL: self.Close()
    if id == OK:
        self.Close()
        doc = c4d.documents.GetActiveDocument()
        wipBd = doc.GetRenderBaseDraw()
        wipCam = wipBd.GetSceneCamera(doc)
        if wipCam is None: wipCam = bd.GetEditorCamera()
        rd = doc.GetActiveRenderData()
        scenesFolder = os.path.join(self.folder, 'WIPScenes')
        if self.backup is True:
            if not os.path.exists(scenesFolder):
                try:
                    os.makedirs(scenesFolder)
                except:
                    return
        wipFolder = os.path.join(self.folder, 'WIPImages')
        if not os.path.exists(wipFolder):
            try:
                os.makedirs(wipFolder)
            except:
                return
        batch = c4d.documents.GetBatchRender()
        try:
            scenes = os.listdir(self.folder)
        except:
            return
        if scenes[0] is None: return
        c4d_files= []
        for file in scenes:
            if file.split('.')[-1] == 'c4d':
                c4d_files.append(file)
        c4d_files.sort(key = alphaNumKey)
        i= 1
        a= 0
        batch.Open()                  
        for c4d_file in c4d_files:
            c4d.StatusSetBar(100*i/len(scenes))
            if i%self.n == 0:
                print(c4d_file)
                
                scene = c4d.documents.LoadDocument(os.path.join(self.folder, c4d_file), c4d.SCENEFILTER_OBJECTS | c4d.SCENEFILTER_MATERIALS)
                rdCont= rd.GetData()
                wipName= rdCont.GetData(c4d.RDATA_PATH)
                wipRd = documents.RenderData()
                wipRd.SetName("WIPBatch")
                wipRd.SetData(rdCont)
                wipPath = os.path.join(wipFolder, wipName + str(a+1))
                wipRd.GetDataInstance().SetFilename(c4d.RDATA_PATH, wipPath)
                scene.InsertRenderData(wipRd, None)
                scene.SetActiveRenderData(wipRd)
                if self.singlecam is True: 
                    cam = wipCam.GetClone()
                    scene.InsertObject(cam)
                    scene.GetRenderBaseDraw().SetSceneCamera(cam)
                if self.backup is True:
                    wipScene = os.path.join(scenesFolder, c4d_file)
                else: wipScene = os.path.join(self.folder, c4d_file)
                documents.SaveDocument(scene, wipScene, c4d.SAVEDOCUMENTFLAGS_DONTADDTORECENTLIST, c4d.FORMAT_C4DEXPORT)
                documents.KillDocument(scene)
                batch.AddFile(wipScene, a)
                batch.EnableElement(a, True)
                a+= 1
            i+= 1     
        c4d.StatusClear()

    return True

def main():
    dlg = Dialog()
    dlg.Open(c4d.DLG_TYPE_MODAL, -1, -1)

if __name__=='__main__':
    main()